@@ -49,7 +49,7 @@ Options to get the internal password representation:
 Options to use the internal password representation:
  -k, --keys <X> <Y> <Z>      Internal password representation as three 32-bits
                               integers in hexadecimal (requires -d, -D, -U,
-                              --change-keys or --bruteforce)
+                              --change-keys, --bruteforce or --mask)
 
  -d, --decipher <file>       File to write the deciphered data (requires -c)
      --keep-header           Write the encryption header at the beginning of
@@ -88,12 +88,21 @@ Options to use the internal password representation:
  -r, --recover-password [ <min>..<max> | <min>.. | ..<max> | <max> ] <charset>
         Shortcut for --length and --bruteforce options
 
+ -m, --mask <mask>
+        Try to recover the password or an equivalent one by generating and
+        testing password candidates following the given mask.
+        The mask is a sequence of fixed characters or character sets (either
+        predefined or custom charsets). Example: -m ?u?l?l?l?l-?d?d?d?d
+
+ -s, --charset <identifier> <charset>
+        Define a custom character set. Example: -s h ?dabcdef
+
      --continue-recovery <checkpoint>
         Starting point of the password recovery. Useful to continue a previous
         non-exhaustive or interrupted password recovery.
 
 Other options:
- -j, --jobs <count>          Number of threads to use for parallelized operations
+ -j, --jobs <count>          Number of threads to use for parallel operations
  -e, --exhaustive            Exhaustively look for all solutions (keys or
                               passwords) instead of stopping after the first
                               solution is found
@@ -302,7 +311,7 @@ try
     }
 
     // recover password
-    if (args.bruteforce)
+    if (args.bruteforce || args.mask)
     {
         std::cout << "[" << put_time << "] Recovering password" << std::endl;
 
@@ -310,13 +319,20 @@ try
 
         const auto [state, restart] = [&]() -> std::pair<Progress::State, std::string>
         {
-            const auto& charset                = *args.bruteforce;
-            const auto& [minLength, maxLength] = args.length.value_or(Arguments::LengthInterval{});
-            auto       start                   = args.recoveryStart;
-            auto       progress                = ConsoleProgress{std::cout};
-            const auto sigintHandler           = SigintHandler{progress.state};
-            passwords = recoverPassword(keysvec.front(), charset, minLength, maxLength, start, args.jobs,
-                                        args.exhaustive, progress);
+            auto       start         = args.recoveryStart;
+            auto       progress      = ConsoleProgress{std::cout};
+            const auto sigintHandler = SigintHandler{progress.state};
+
+            if (args.bruteforce)
+            {
+                const auto& charset                = *args.bruteforce;
+                const auto& [minLength, maxLength] = args.length.value_or(Arguments::LengthInterval{});
+                passwords = recoverPassword(keysvec.front(), charset, minLength, maxLength, start, args.jobs,
+                                            args.exhaustive, progress);
+            }
+            else
+                passwords = recoverPassword(keysvec.front(), *args.mask, start, args.jobs, args.exhaustive, progress);
+
             return {progress.state, start};
         }();
 
